package core

import (
	"git.akyoto.dev/cli/q/src/build/asm"
	"git.akyoto.dev/cli/q/src/build/config"
	"git.akyoto.dev/cli/q/src/build/cpu"
)

// SaveRegister attempts to move a variable occupying this register to another register.
func (f *Function) SaveRegister(register cpu.Register) {
	if !f.cpu.IsUsed(register) {
		return
	}

	for _, general := range f.cpu.General {
		if register == general {
			return
		}
	}

	var variable *Variable

	for _, v := range f.variables {
		if v.Register == register {
			variable = v
			break
		}
	}

	if variable == nil || variable.Alive == 0 {
		return
	}

	newRegister := f.cpu.MustFindFree(f.cpu.General)
	f.cpu.Reserve(newRegister)

	if config.Comments {
		f.Comment("save %s to %s", register, newRegister)
	}

	f.RegisterRegister(asm.MOVE, newRegister, register)
	variable.Register = newRegister
}
